import uuid

from typing import List

import barker.schemas.customer as schemas

from fastapi import APIRouter, Depends, Security
from sqlalchemy.orm import Session

from ..core.security import get_current_active_user as get_user
from ..db.session import SessionLocal
from ..models.customer import Customer
from ..models.db_setting import DbSetting
from ..models.sale_category import SaleCategory
from ..schemas.user_token import UserToken


router = APIRouter()


# Dependency
def get_db():
    try:
        db = SessionLocal()
        yield db
    finally:
        db.close()


@router.get("", response_model=List[schemas.DiscountItem])
def show_blank(
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["customers"]),
) -> List[schemas.DiscountItem]:
    return customer_discounts_blank(db)


@router.get("/{id_}", response_model=List[schemas.DiscountItem])
def show_id(
    id_: uuid.UUID,
    db: Session = Depends(get_db),
    user: UserToken = Security(get_user, scopes=["customers"]),
) -> List[schemas.DiscountItem]:
    item: Customer = db.query(Customer).filter(Customer.id == id_).first()
    return customer_discounts(item, db)


def customer_discounts(item: Customer, db: Session) -> List[schemas.DiscountItem]:
    default_discount = db.query(DbSetting).filter(DbSetting.name == "Prefill Customer Discount").first().data["Value"]

    return [
        schemas.DiscountItem(
            id=sc.id,
            name=sc.name,
            discount=0
            if not default_discount
            else next((d.discount for d in item.discounts if d.sale_category_id == sc.id), 0),
            discountLimit=sc.discount_limit,
            customerDiscount=next((d.discount for d in item.discounts if d.sale_category_id == sc.id), 0),
        )
        for sc in db.query(SaleCategory).filter(SaleCategory.discount_limit != 0).order_by(SaleCategory.name).all()
    ]


def customer_discounts_blank(db: Session) -> List[schemas.DiscountItem]:
    return [
        schemas.DiscountItem(
            id=sc.id,
            name=sc.name,
            discount=0,
            discountLimit=sc.discount_limit,
            customerDiscount=0,
        )
        for sc in db.query(SaleCategory).filter(SaleCategory.discount_limit != 0).order_by(SaleCategory.name).all()
    ]
